==> open-xchange-core_7.6.3.orig.tar.bz2: open-xchange-core-7.6.3/com.openexchange.html/src/com/openexchange/html/tools/CombinedCharSequence.java <==
/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the OX Software GmbH group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.html.tools;

import java.util.Arrays;
import java.util.Collection;
import java.util.LinkedList;
import java.util.List;

/**
 * {@link CombinedCharSequence}
 *
 * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
 * @since v7.8.3
 */
public class CombinedCharSequence implements CharSequence {

    private final Collection<CharSequence> sequences;

    /**
     * Initializes a new {@link CombinedCharSequence}.
     */
    public CombinedCharSequence(CharSequence... sequences) {
        super();
        this.sequences = Arrays.asList(sequences);
    }

    /**
     * Initializes a new {@link CombinedCharSequence}.
     */
    CombinedCharSequence(Collection<CharSequence> sequences) {
        super();
        this.sequences = sequences;
    }


    @Override
    public int length() {
        int length = 0;
        for (CharSequence sequence : sequences) {
            length += sequence.length();
        }
        return length;
    }

    @Override
    public char charAt(int index) {
        int start = 0;
        int end;
        for (CharSequence sequence : sequences) {
            end = sequence.length() + start;
            if (index >= start && index < end) {
                return sequence.charAt(index - start);
            }
            start = end;
        }
        throw new IndexOutOfBoundsException("Index out of bounds: " + index);
    }

    @Override
    public CharSequence subSequence(int st, int en) {
        List<CharSequence> seqs = null;

        int start = 0;
        int end;
        for (CharSequence sequence : sequences) {
            end = sequence.length() + start;
            if (st >= start && en <= end) {
                return sequence.subSequence(st - start, en - start);
            }
            if (st >= start && st < end && en > end) {
                seqs = new LinkedList<CharSequence>();
                seqs.add(sequence.subSequence(st - start, end));
            } else if (st < start && st < end && en <= end) {
                seqs.add(sequence.subSequence(0, en - start));
            } else if (st < start && st < end && en > end) {
                seqs.add(sequence);
            }
            start = end;
        }

        return new CombinedCharSequence(seqs);
    }

    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder(sequences.size() << 4);
        for (CharSequence sequence : sequences) {
            sb.append(sequence);
        }
        return sb.toString();
    }

}

==> open-xchange-core_7.8.4.orig.tar.bz2: open-xchange-core-7.8.4/com.openexchange.html/src/com/openexchange/html/tools/CombinedCharSequence.java <==
/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the OX Software GmbH group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.html.tools;

import java.util.Arrays;
import java.util.Collection;
import java.util.LinkedList;
import java.util.List;

/**
 * {@link CombinedCharSequence}
 *
 * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
 * @since v7.8.3
 */
public class CombinedCharSequence implements CharSequence {

    private final Collection<CharSequence> sequences;

    /**
     * Initializes a new {@link CombinedCharSequence}.
     */
    public CombinedCharSequence(CharSequence... sequences) {
        super();
        this.sequences = Arrays.asList(sequences);
    }

    /**
     * Initializes a new {@link CombinedCharSequence}.
     */
    CombinedCharSequence(Collection<CharSequence> sequences) {
        super();
        this.sequences = sequences;
    }


    @Override
    public int length() {
        int length = 0;
        for (CharSequence sequence : sequences) {
            length += sequence.length();
        }
        return length;
    }

    @Override
    public char charAt(int index) {
        int start = 0;
        int end;
        for (CharSequence sequence : sequences) {
            end = sequence.length() + start;
            if (index >= start && index < end) {
                return sequence.charAt(index - start);
            }
            start = end;
        }
        throw new IndexOutOfBoundsException("Index out of bounds: " + index);
    }

    @Override
    public CharSequence subSequence(int st, int en) {
        List<CharSequence> seqs = null;

        int start = 0;
        int end;
        for (CharSequence sequence : sequences) {
            end = sequence.length() + start;
            if (st >= start && en <= end) {
                return sequence.subSequence(st - start, en - start);
            }
            if (st >= start && st < end && en > end) {
                seqs = new LinkedList<>();
                seqs.add(sequence.subSequence(st - start, end));
            } else if (st < start && st < end && en <= end) {
                seqs.add(sequence.subSequence(0, en - start));
            } else if (st < start && st < end && en > end) {
                seqs.add(sequence);
            }
            start = end;
        }

        return new CombinedCharSequence(seqs);
    }

    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder(sequences.size() << 4);
        for (CharSequence sequence : sequences) {
            sb.append(sequence);
        }
        return sb.toString();
    }

}
